"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.productArrayValidate = exports.productValidate = exports.Product = exports.Size = exports.Color = void 0;
require("reflect-metadata");
const chai_1 = require("chai");
const class_fixtures_factory_1 = require("class-fixtures-factory");
const class_transformer_1 = require("class-transformer");
const class_validator_1 = require("class-validator");
const faker_1 = __importDefault(require("faker"));
const src_1 = require("../../src");
const validate_1 = require("../factories/validate");
class Color {
    constructor() {
        this.name = '';
    }
}
__decorate([
    src_1.ArrayMember(0),
    class_fixtures_factory_1.Fixture(() => faker_1.default.commerce.color()),
    __metadata("design:type", String)
], Color.prototype, "name", void 0);
exports.Color = Color;
class Size {
    constructor() {
        this.size = 0;
    }
}
__decorate([
    src_1.ArrayMember(0),
    class_fixtures_factory_1.Fixture(() => faker_1.default.random.number()),
    __metadata("design:type", Number)
], Size.prototype, "size", void 0);
exports.Size = Size;
class Product {
    constructor() {
        this.id = 0;
        this.price = 0;
        this.displayPrice = '0';
        this.sizes = [];
        this.values = [];
    }
}
__decorate([
    src_1.ArrayMember(0),
    class_validator_1.IsNumber(),
    class_fixtures_factory_1.Fixture(() => faker_1.default.random.number()),
    __metadata("design:type", Number)
], Product.prototype, "id", void 0);
__decorate([
    src_1.ArrayMember(1),
    class_validator_1.IsNumber(),
    class_fixtures_factory_1.Fixture(() => faker_1.default.random.number()),
    __metadata("design:type", Number)
], Product.prototype, "price", void 0);
__decorate([
    src_1.ArrayMember(2),
    class_validator_1.IsString(),
    class_transformer_1.Transform((v) => v === null || v === void 0 ? void 0 : v.toString()),
    class_fixtures_factory_1.Fixture(() => faker_1.default.random.number().toString()),
    __metadata("design:type", String)
], Product.prototype, "displayPrice", void 0);
__decorate([
    src_1.ArrayMember(3),
    class_validator_1.ValidateNested(),
    class_transformer_1.Type(() => Color),
    class_fixtures_factory_1.Fixture({ type: () => Color }),
    __metadata("design:type", Color)
], Product.prototype, "color", void 0);
__decorate([
    src_1.ArrayMember(4, { isArray: true }),
    class_validator_1.ValidateNested({ each: true }),
    class_transformer_1.Type(() => Size),
    class_fixtures_factory_1.Fixture({ type: () => [Size] }),
    __metadata("design:type", Array)
], Product.prototype, "sizes", void 0);
__decorate([
    src_1.ArrayMember(5),
    class_validator_1.IsNumber(undefined, { each: true }),
    class_fixtures_factory_1.Fixture({ type: () => [Number] }),
    __metadata("design:type", Array)
], Product.prototype, "values", void 0);
exports.Product = Product;
function productValidate(expected, result, options = validate_1.defaultValidateOptions) {
    var _a;
    chai_1.expect(result).not.null;
    if (options.constructor) {
        chai_1.expect(result).property('constructor', Product);
    }
    chai_1.expect(result).property('id', expected.id);
    chai_1.expect(result).property('price', expected.price);
    // class transform
    chai_1.expect(result).property('displayPrice', expected.displayPrice);
    // nested
    if (expected.color) {
        if (options.constructor) {
            chai_1.expect(result).property('color').property('constructor', Color);
        }
        chai_1.expect(result).property('color').property('name', expected.color.name);
    }
    // nested array
    chai_1.expect(result).property('sizes').property('constructor', Array);
    chai_1.expect(result).property('sizes').length(expected.sizes.length);
    for (const [i, { size }] of expected.sizes.entries()) {
        if (options.constructor) {
            chai_1.expect(result).property('sizes').property(i.toString()).property('constructor', Size);
        }
        chai_1.expect(result).property('sizes').property(i.toString()).property('size', size);
    }
    // array values
    chai_1.expect(result).property('values').property('constructor', Array);
    chai_1.expect(result).property('values').length(expected.values.length);
    for (const [i, v] of (_a = expected.values.entries()) !== null && _a !== void 0 ? _a : []) {
        chai_1.expect(result).property('values').property(i.toString(), v);
    }
}
exports.productValidate = productValidate;
function productArrayValidate(expected, result) {
    var _a;
    chai_1.expect(result).not.null;
    chai_1.expect(result).property('constructor', Array);
    chai_1.expect(result).property('0', expected.id);
    chai_1.expect(result).property('1', expected.price);
    // class transform
    chai_1.expect(result).property('2', expected.displayPrice);
    // nested
    if (expected.color) {
        chai_1.expect(result).property('3').property('0', expected.color.name);
    }
    else {
        chai_1.expect((result === null || result === void 0 ? void 0 : result[3]) == null).true;
    }
    // nested array
    chai_1.expect(result).property('4').property('constructor', Array);
    chai_1.expect(result).property('4').length(expected.sizes.length);
    for (const [i, { size }] of expected.sizes.entries()) {
        chai_1.expect(result).property('4').property(i.toString()).length(1);
        chai_1.expect(result).property('4').property(i.toString()).property('0', size);
    }
    // array values
    chai_1.expect(result).property('5').property('constructor', Array);
    chai_1.expect(result).property('5').length(expected.values.length);
    for (const [i, v] of (_a = expected.values.entries()) !== null && _a !== void 0 ? _a : []) {
        chai_1.expect(result).property('5').property(i.toString(), v);
    }
}
exports.productArrayValidate = productArrayValidate;
//# sourceMappingURL=Product.js.map